﻿// Example of asynchronous programmings in C# and .NET 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Diagnostics;
using System.Net.Http;
using System.Threading;
using System.Net;
using System.IO;

namespace Asynch
{
    class Asynch
    {
        private static string downloadPage(string url)
        {
            WebClient client = new WebClient();
            System.Console.WriteLine("downloadPage, url = {0}, starting", url);
            string html = client.DownloadString(url);
            System.Console.WriteLine("downloadPage, url = {0}, done", url);
            return html;
        }

        private static string[] downloadPages(string[] urls)
        {
            System.Console.WriteLine("downloadPages, urls length = {0}, starting", urls.Length);
            string[] result = new string[urls.Length];
            for (int i = 0; i < urls.Length; i++)
            {
                result[i] = downloadPage(urls[i]);
            }

            System.Console.WriteLine("downloadPages, urls length = {0}, done", urls.Length);
            return result;
        }

        private static async Task<string> downloadPageAsync(string url)
        {
            WebClient client = new WebClient();
            System.Console.WriteLine("downloadPageAsync, url = {0}, starting, thread = {1}", url, Thread.CurrentThread.ManagedThreadId);
            string html = await client.DownloadStringTaskAsync(url);
            System.Console.WriteLine("downloadPageAsync, url = {0}, done, thread = {1}", url, Thread.CurrentThread.ManagedThreadId);
            return html;
        }

        private static async Task<string[]> downloadPagesAsync(string[] urls)
        {
            System.Console.WriteLine("downloadPagesAsync, urls length = {0}, starting, thread = {1}", urls.Length, Thread.CurrentThread.ManagedThreadId);
            string[] result = new string[urls.Length];
            for (int i = 0; i < urls.Length; i++)
            {
                result[i] = await downloadPageAsync(urls[i]);
            }

            System.Console.WriteLine("downloadPagesAsync, urls length = {0}, done, thread = {1}", urls.Length, Thread.CurrentThread.ManagedThreadId);
            return result;
        }

        static void Main(string[] args)
        {
            string[] urls = {"http://google.com",
                              "http://cnn.com",
                              "http://news.bbc.co.uk",
                              "http://wired.com",
                              "http://mtech.edu",
                              "http://keithv.com"};

            Task<string[]> results = downloadPagesAsync(urls);
            
            // Print dots until all the pages are loaded
            while (!results.IsCompleted)
            {
                Thread.Sleep(100);
                System.Console.Write(".");
            }

            System.Console.WriteLine();
            System.Console.WriteLine("All the pages are loaded!");

            // Display the URL and the loaded length of each page
            string[] pages = results.Result;
            for (int i = 0; i < urls.Length; i++)
            {
                System.Console.WriteLine("URL {0}, len = {1}", urls[i], pages[i].Length);
            }

        }
    }
}
